=== modified file 'drizzled/field.cc'
--- drizzled/field.cc	2009-02-21 01:15:13 +0000
+++ drizzled/field.cc	2009-02-26 20:57:39 +0000
@@ -1606,6 +1606,13 @@
       /* Allow empty as default value. */
       String str,*res;
       res= fld_default_value->val_str(&str);
+      if (res->length())
+      {
+        my_error(ER_BLOB_CANT_HAVE_DEFAULT, MYF(0),
+                 fld_name);
+        return(true);
+      }
+
     }
     flags|= BLOB_FLAG;
     break;
@@ -1776,7 +1783,8 @@
 }
 
 
-Field *make_field(TABLE_SHARE *share, unsigned char *ptr, uint32_t field_length,
+Field *make_field(TABLE_SHARE *share, MEM_ROOT *root,
+                  unsigned char *ptr, uint32_t field_length,
 		  unsigned char *null_pos, unsigned char null_bit,
 		  uint32_t pack_flag,
 		  enum_field_types field_type,
@@ -1785,6 +1793,10 @@
 		  TYPELIB *interval,
 		  const char *field_name)
 {
+
+  if(!root)
+    root= current_mem_root();
+
   if (!f_maybe_null(pack_flag))
   {
     null_pos=0;
@@ -1809,7 +1821,7 @@
     if (!f_is_packed(pack_flag))
     {
       if (field_type == DRIZZLE_TYPE_VARCHAR)
-        return new Field_varstring(ptr,field_length,
+        return new (root) Field_varstring(ptr,field_length,
                                    HA_VARCHAR_PACKLENGTH(field_length),
                                    null_pos,null_bit,
                                    unireg_check, field_name,
@@ -1823,30 +1835,30 @@
 				      field_length);
 
     if (f_is_blob(pack_flag))
-      return new Field_blob(ptr,null_pos,null_bit,
+      return new (root) Field_blob(ptr,null_pos,null_bit,
 			    unireg_check, field_name, share,
 			    pack_length, field_charset);
     if (interval)
     {
       if (f_is_enum(pack_flag))
-{
-	return new Field_enum(ptr,field_length,null_pos,null_bit,
+      {
+        return new (root) Field_enum(ptr,field_length,null_pos,null_bit,
 				  unireg_check, field_name,
 				  get_enum_pack_length(interval->count),
                                   interval, field_charset);
-}
+      }
     }
   }
 
   switch (field_type) {
   case DRIZZLE_TYPE_NEWDECIMAL:
-    return new Field_new_decimal(ptr,field_length,null_pos,null_bit,
+    return new (root) Field_new_decimal(ptr,field_length,null_pos,null_bit,
                                  unireg_check, field_name,
                                  f_decimals(pack_flag),
                                  f_is_decimal_precision(pack_flag) != 0,
                                  f_is_dec(pack_flag) == 0);
   case DRIZZLE_TYPE_DOUBLE:
-    return new Field_double(ptr,field_length,null_pos,null_bit,
+    return new (root) Field_double(ptr,field_length,null_pos,null_bit,
 			    unireg_check, field_name,
 			    f_decimals(pack_flag),
 			    false,
@@ -1854,30 +1866,30 @@
   case DRIZZLE_TYPE_TINY:
     assert(0);
   case DRIZZLE_TYPE_LONG:
-    return new Field_long(ptr,field_length,null_pos,null_bit,
+    return new (root) Field_long(ptr,field_length,null_pos,null_bit,
 			   unireg_check, field_name,
                            false,
 			   f_is_dec(pack_flag) == 0);
   case DRIZZLE_TYPE_LONGLONG:
-    return new Field_int64_t(ptr,field_length,null_pos,null_bit,
+    return new (root) Field_int64_t(ptr,field_length,null_pos,null_bit,
 			      unireg_check, field_name,
                               false,
 			      f_is_dec(pack_flag) == 0);
   case DRIZZLE_TYPE_TIMESTAMP:
-    return new Field_timestamp(ptr,field_length, null_pos, null_bit,
+    return new (root) Field_timestamp(ptr,field_length, null_pos, null_bit,
                                unireg_check, field_name, share,
                                field_charset);
   case DRIZZLE_TYPE_DATE:
-    return new Field_date(ptr,null_pos,null_bit,
+    return new (root) Field_date(ptr,null_pos,null_bit,
 			     unireg_check, field_name, field_charset);
   case DRIZZLE_TYPE_TIME:
-    return new Field_time(ptr,null_pos,null_bit,
+    return new (root) Field_time(ptr,null_pos,null_bit,
 			  unireg_check, field_name, field_charset);
   case DRIZZLE_TYPE_DATETIME:
-    return new Field_datetime(ptr,null_pos,null_bit,
+    return new (root) Field_datetime(ptr,null_pos,null_bit,
 			      unireg_check, field_name, field_charset);
   case DRIZZLE_TYPE_NULL:
-    return new Field_null(ptr, field_length, unireg_check, field_name,
+    return new (root) Field_null(ptr, field_length, unireg_check, field_name,
                           field_charset);
   case DRIZZLE_TYPE_VIRTUAL:                    // Must not happen
     assert(0);
@@ -1934,7 +1946,7 @@
   def=0;
   char_length= length;
 
-  if (!(flags & (NO_DEFAULT_VALUE_FLAG | BLOB_FLAG)) &&
+  if (!(flags & (NO_DEFAULT_VALUE_FLAG )) &&
       old_field->ptr && orig_field &&
       (sql_type != DRIZZLE_TYPE_TIMESTAMP ||                /* set def only if */
        old_field->table->timestamp_field != old_field ||  /* timestamp field */

=== modified file 'drizzled/field.h'
--- drizzled/field.h	2009-02-17 14:41:01 +0000
+++ drizzled/field.h	2009-02-25 16:45:57 +0000
@@ -62,6 +62,9 @@
   void operator=(Field &);
 public:
   static void *operator new(size_t size) {return sql_alloc(size); }
+  static void *operator new(size_t size, MEM_ROOT *mem_root)
+  { return (void*) alloc_root(mem_root, (uint32_t) size); }
+
   static void operator delete(void *, size_t)
   { TRASH(ptr_arg, size); }
 
@@ -669,7 +672,7 @@
 };
 
 
-Field *make_field(TABLE_SHARE *share, unsigned char *ptr, uint32_t field_length,
+Field *make_field(TABLE_SHARE *share, MEM_ROOT *root, unsigned char *ptr, uint32_t field_length,
                   unsigned char *null_pos, unsigned char null_bit,
                   uint32_t pack_flag, enum_field_types field_type,
                   const CHARSET_INFO * cs,

=== modified file 'drizzled/serialize/table.proto'
--- drizzled/serialize/table.proto	2009-02-19 16:32:29 +0000
+++ drizzled/serialize/table.proto	2009-02-26 04:31:12 +0000
@@ -76,18 +76,17 @@
     enum FieldType {
       DOUBLE = 0;
       VARCHAR = 1;
-      TEXT = 2;
-      BLOB = 3;
-      ENUM = 4;
-      INTEGER = 5;
-      BIGINT = 6;
-      DECIMAL = 7;
-      DATE = 8;
-      TIME = 9;
-      TIMESTAMP = 10;
-      DATETIME = 11;
-      TINYINT = 12;
-      VIRTUAL = 13;
+      BLOB = 2;
+      ENUM = 3;
+      INTEGER = 4;
+      BIGINT = 5;
+      DECIMAL = 6;
+      DATE = 7;
+      TIME = 8;
+      TIMESTAMP = 9;
+      DATETIME = 10;
+      TINYINT = 11;
+      VIRTUAL = 12;
     }
 
     enum FieldFormatType {
@@ -99,6 +98,10 @@
     message FieldOptions {
       optional string default_value = 1;
       optional string update_value = 2;
+      optional bool default_null = 3 [default = false];
+      optional bytes default_bin_value = 4;
+
+      optional int32 length = 100; /* TODO: should go away */
     }
 
     message TimestampFieldOptions {
@@ -127,7 +130,9 @@
 
     message SetFieldOptions {
       required int32 count_elements = 1;
-      repeated string field_value = 2;
+      optional uint32 collation_id = 2;
+      optional string collation = 3;
+      repeated string field_value = 4;
     }
 
     message VirtualFieldOptions {
@@ -149,6 +154,8 @@
     optional string comment = 16; /* Reserve 0-15 for frequently accessed attributes */
     optional SetFieldOptions set_options = 17;
     optional TimestampFieldOptions timestamp_options = 18;
+
+    optional uint32 pack_flag = 100; /* MUST DIE */
   }
 
   message Index {
@@ -166,6 +173,8 @@
       required uint32 fieldnr = 1;
       optional int32 compare_length = 2;
       optional bool in_reverse_order = 3 [default = false];
+
+      optional uint32 key_type = 101; /* THIS MUST DIE. Along with pack_flag*/
     }
 
     message IndexOptions {

=== modified file 'drizzled/serialize/table_reader.cc'
--- drizzled/serialize/table_reader.cc	2009-02-18 16:48:59 +0000
+++ drizzled/serialize/table_reader.cc	2009-02-25 16:46:28 +0000
@@ -23,7 +23,16 @@
 void print_field(const ::drizzle::Table::Field &field)
 {
   cout << "\t`" << field.name() << "`";
-  switch (field.type())
+
+  Table::Field::FieldType field_type= field.type();
+
+  if(field_type==Table::Field::VIRTUAL)
+  {
+    cout << " VIRTUAL"; // FIXME
+    field_type= field.virtual_options().type();
+  }
+
+  switch (field_type)
   {
     case Table::Field::DOUBLE:
     cout << " DOUBLE ";
@@ -31,11 +40,10 @@
   case Table::Field::VARCHAR:
     cout << " VARCHAR(" << field.string_options().length() << ")";
     break;
-  case Table::Field::TEXT:
-    cout << " TEXT ";
-    break;
   case Table::Field::BLOB:
-    cout << " BLOB ";
+    cout << " BLOB "; /* FIXME: or text, depends on collation */
+    if(field.string_options().has_collation_id())
+      cout << "COLLATION=" << field.string_options().collation_id() << " ";
     break;
   case Table::Field::ENUM:
     {
@@ -78,8 +86,14 @@
     cout << " DATETIME ";
     break;
   case Table::Field::VIRTUAL:
-    cout << " VIRTUAL"; // FIXME
-    break;
+    abort(); // handled above.
+  }
+
+  if(field.type()==Table::Field::VIRTUAL)
+  {
+    cout << " AS (" << field.virtual_options().expression() << ") ";
+    if(field.virtual_options().physically_stored())
+      cout << " STORED ";
   }
 
   if (field.type() == Table::Field::INTEGER
@@ -100,7 +114,7 @@
 	 && field.constraints().is_nullable()))
     cout << " NOT NULL ";
 
-  if (field.type() == Table::Field::TEXT
+  if (field.type() == Table::Field::BLOB
       || field.type() == Table::Field::VARCHAR)
   {
     if (field.string_options().has_collation())
@@ -110,6 +124,16 @@
   if (field.options().has_default_value())
     cout << " DEFAULT `" << field.options().default_value() << "` " ;
 
+  if (field.options().has_default_bin_value())
+  {
+    string v= field.options().default_bin_value();
+    cout << " DEFAULT 0x";
+    for(unsigned int i=0; i< v.length(); i++)
+    {
+      printf("%.2x", *(v.c_str()+i));
+    }
+  }
+
   if (field.type() == Table::Field::TIMESTAMP)
     if (field.timestamp_options().has_auto_updates()
       && field.timestamp_options().auto_updates())

=== modified file 'drizzled/sql_table.cc'
--- drizzled/sql_table.cc	2009-02-21 05:34:53 +0000
+++ drizzled/sql_table.cc	2009-02-26 20:57:39 +0000
@@ -4018,7 +4018,7 @@
     */
   Field **f_ptr,*field;
   for (f_ptr=table->field ; (field= *f_ptr) ; f_ptr++)
-    {
+  {
     /* Check if field should be dropped */
     Alter_drop *drop;
     drop_it.rewind();
@@ -4026,19 +4026,19 @@
     {
       if (drop->type == Alter_drop::COLUMN &&
 	  !my_strcasecmp(system_charset_info,field->field_name, drop->name))
-    {
+      {
 	/* Reset auto_increment value if it was dropped */
 	if (MTYP_TYPENR(field->unireg_check) == Field::NEXT_NUMBER &&
 	    !(used_fields & HA_CREATE_USED_AUTO))
-      {
+	{
 	  create_info->auto_increment_value=0;
 	  create_info->used_fields|=HA_CREATE_USED_AUTO;
+	}
+	break;
       }
-	break;
     }
-  }
     if (drop)
-      {
+    {
       drop_it.remove();
       continue;
     }
@@ -4061,13 +4061,13 @@
         goto err;
       }
       if (!def->after)
-	{
+      {
 	new_create_list.push_back(def);
 	def_it.remove();
-	}
       }
-      else
-      {
+    }
+    else
+    {
       /*
         This field was not dropped and not changed, add it to the list
         for the new table.
@@ -4077,12 +4077,12 @@
       alter_it.rewind();			// Change default if ALTER
       Alter_column *alter;
       while ((alter=alter_it++))
-        {
+      {
 	if (!my_strcasecmp(system_charset_info,field->field_name, alter->name))
 	  break;
-        }
+      }
       if (alter)
-	{
+      {
 	if (def->sql_type == DRIZZLE_TYPE_BLOB)
 	{
 	  my_error(ER_BLOB_CANT_HAVE_DEFAULT, MYF(0), def->change);
@@ -4104,21 +4104,21 @@
       my_error(ER_BAD_FIELD_ERROR, MYF(0), def->change, table->s->table_name.str);
       goto err;
     }
-      /*
+    /*
       Check that the DATE/DATETIME not null field we are going to add is
       either has a default value or the '0000-00-00' is allowed by the
       set sql mode.
       If the '0000-00-00' value isn't allowed then raise the error_if_not_empty
       flag to allow ALTER Table only if the table to be altered is empty.
-      */
+    */
     if ((def->sql_type == DRIZZLE_TYPE_DATE ||
          def->sql_type == DRIZZLE_TYPE_DATETIME) &&
-         !alter_info->datetime_field &&
-         !(~def->flags & (NO_DEFAULT_VALUE_FLAG | NOT_NULL_FLAG)) &&
-         session->variables.sql_mode & MODE_NO_ZERO_DATE)
+	!alter_info->datetime_field &&
+	!(~def->flags & (NO_DEFAULT_VALUE_FLAG | NOT_NULL_FLAG)) &&
+	session->variables.sql_mode & MODE_NO_ZERO_DATE)
     {
-        alter_info->datetime_field= def;
-        alter_info->error_if_not_empty= true;
+      alter_info->datetime_field= def;
+      alter_info->error_if_not_empty= true;
     }
     if (!def->after)
       new_create_list.push_back(def);
@@ -4132,12 +4132,12 @@
       {
 	if (!my_strcasecmp(system_charset_info,def->after, find->field_name))
 	  break;
-  }
+      }
       if (!find)
-  {
+      {
 	my_error(ER_BAD_FIELD_ERROR, MYF(0), def->after, table->s->table_name.str);
-    goto err;
-  }
+	goto err;
+      }
       find_it.after(def);			// Put element after this
       /*
         XXX: hack for Bug#28427.
@@ -4162,32 +4162,32 @@
     my_error(ER_BAD_FIELD_ERROR, MYF(0),
              alter_info->alter_list.head()->name, table->s->table_name.str);
     goto err;
-    }
+  }
   if (!new_create_list.elements)
-    {
+  {
     my_message(ER_CANT_REMOVE_ALL_FIELDS, ER(ER_CANT_REMOVE_ALL_FIELDS),
                MYF(0));
     goto err;
-    }
+  }
 
-    /*
+  /*
     Collect all keys which isn't in drop list. Add only those
     for which some fields exists.
-    */
+  */
 
   for (uint32_t i=0 ; i < table->s->keys ; i++,key_info++)
-    {
+  {
     char *key_name= key_info->name;
     Alter_drop *drop;
     drop_it.rewind();
     while ((drop=drop_it++))
-      {
+    {
       if (drop->type == Alter_drop::KEY &&
 	  !my_strcasecmp(system_charset_info,key_name, drop->name))
 	break;
-      }
+    }
     if (drop)
-        {
+    {
       drop_it.remove();
       continue;
     }
@@ -4202,9 +4202,9 @@
       Create_field *cfield;
       field_it.rewind();
       while ((cfield=field_it++))
-    {
+      {
 	if (cfield->change)
-    {
+	{
 	  if (!my_strcasecmp(system_charset_info, key_part_name,
 			     cfield->change))
 	    break;

=== modified file 'drizzled/table.cc'
--- drizzled/table.cc	2009-02-26 02:10:12 +0000
+++ drizzled/table.cc	2009-02-26 20:57:39 +0000
@@ -37,6 +37,13 @@
 #include <drizzled/unireg.h>
 #include <drizzled/serialize/table.pb.h>
 
+#include <drizzled/item/string.h>
+#include <drizzled/item/int.h>
+#include <drizzled/item/decimal.h>
+#include <drizzled/item/float.h>
+#include <drizzled/item/null.h>
+
+
 using namespace std;
 
 /* Keyword for parsing virtual column functions */
@@ -327,12 +334,80 @@
     field_type= DRIZZLE_TYPE_VIRTUAL;
     break;
   default:
-    abort();
+    assert(1);
   }
 
   return field_type;
 }
 
+Item * default_value_item(enum_field_types field_type,
+			  const CHARSET_INFO *charset,
+			  bool default_null, string default_value,
+			  string default_bin_value)
+{
+  Item *default_item= NULL;
+  int error= 0;
+
+  if(default_null)
+  {
+    return new Item_null();
+  }
+
+  switch(field_type)
+  {
+  case DRIZZLE_TYPE_TINY:
+  case DRIZZLE_TYPE_LONG:
+  case DRIZZLE_TYPE_LONGLONG:
+    default_item= new Item_int(default_value.c_str(),
+			       (int64_t) my_strtoll10(default_value.c_str(),
+						      NULL,
+						      &error),
+			       default_value.length());
+    break;
+  case DRIZZLE_TYPE_DOUBLE:
+    default_item= new Item_float(default_value.c_str(), default_value.length());
+    break;
+  case DRIZZLE_TYPE_NULL:
+    return new Item_null();
+    break;
+  case DRIZZLE_TYPE_TIMESTAMP:
+  case DRIZZLE_TYPE_TIME:
+  case DRIZZLE_TYPE_DATETIME:
+  case DRIZZLE_TYPE_DATE:
+    if(default_value.compare("NOW()")==0)
+      break;
+  case DRIZZLE_TYPE_ENUM:
+    default_item= new Item_string(default_value.c_str(),
+				  default_value.length(),
+				  system_charset_info);
+    break;
+  case DRIZZLE_TYPE_VARCHAR:
+  case DRIZZLE_TYPE_BLOB: /* Blob is here due to TINYTEXT. Feel the hate. */
+    if(charset==&my_charset_bin)
+    {
+      default_item= new Item_string(default_bin_value.c_str(),
+				    default_bin_value.length(),
+				    &my_charset_bin);
+    }
+    else
+    {
+      default_item= new Item_string(default_value.c_str(),
+				    default_value.length(),
+				    system_charset_info);
+    }
+    break;
+  case DRIZZLE_TYPE_VIRTUAL:
+    break;
+  case DRIZZLE_TYPE_NEWDECIMAL:
+    default_item= new Item_decimal(default_value.c_str(),
+				   default_value.length(),
+				   system_charset_info);
+    break;
+  }
+
+  return default_item;
+}
+
 int parse_table_proto(Session *session, drizzle::Table &table, TABLE_SHARE *share)
 {
   {
@@ -508,9 +583,8 @@
       *rec_per_key++=0;
 
       key_part->field= NULL;
-      key_part->fieldnr= part.fieldnr();
+      key_part->fieldnr= part.fieldnr() + 1; // start from 1.
       key_part->null_bit= 0;
-      /* key_part->offset= ASS ASS ASS. Set later in the frm code */
       /* key_part->null_offset is only set if null_bit (see later) */
       /* key_part->key_type= */ /* I *THINK* this may be okay.... */
       /* key_part->type ???? */
@@ -521,6 +595,10 @@
       key_part->length= part.compare_length();
 
       key_part->store_length= key_part->length;
+
+      /* key_part->offset is set later */
+      key_part->key_type= part.key_type();
+
     }
 
     if(!indx.has_comment())
@@ -573,9 +651,10 @@
   share->vfields= 0;
   share->stored_fields= share->fields;
 
-/*  share->field= (Field**) alloc_root(&share->mem_root,
-				     (share->fields * sizeof(Field*)));
-*/
+  share->field= (Field**) alloc_root(&share->mem_root,
+				     ((share->fields+1) * sizeof(Field*)));
+  share->field[share->fields]= NULL;
+
   uint32_t null_fields= 0;
   share->reclength= 0;
 
@@ -584,13 +663,18 @@
 
   assert(field_offsets && field_pack_length); // TODO: fixme
 
+  uint32_t interval_count= 0;
+  uint32_t interval_parts= 0;
+
+  uint32_t stored_columns_reclength= 0;
+
   for(unsigned int fieldnr=0; fieldnr < share->fields; fieldnr++)
   {
     drizzle::Table::Field pfield= table.field(fieldnr);
     if(pfield.has_constraints() && pfield.constraints().is_nullable())
       null_fields++;
 
-    field_offsets[fieldnr]= share->reclength;
+    bool field_is_stored= true;
 
     enum_field_types drizzle_field_type=
       proto_field_type_to_drizzle_type(pfield.type());
@@ -601,8 +685,12 @@
 	pfield.virtual_options();
 
       drizzle_field_type=proto_field_type_to_drizzle_type(field_options.type());
+
+      field_is_stored= field_options.physically_stored();
     }
 
+    field_offsets[fieldnr]= stored_columns_reclength;
+
     /* the below switch is very similar to
        Create_field::create_length_to_internal_length in field.cc
        (which should one day be replace by just this code)
@@ -634,6 +722,9 @@
 
 	field_pack_length[fieldnr]=
 	  get_enum_pack_length(field_options.field_value_size());
+
+	interval_count++;
+	interval_parts+= field_options.field_value_size();
       }
       break;
     case DRIZZLE_TYPE_NEWDECIMAL:
@@ -650,6 +741,37 @@
     }
 
     share->reclength+= field_pack_length[fieldnr];
+
+    if(field_is_stored)
+      stored_columns_reclength+= field_pack_length[fieldnr];
+  }
+
+  /* data_offset added to stored_rec_length later */
+  share->stored_rec_length= stored_columns_reclength;
+
+  /* fix up offsets for non-stored fields (at end of record) */
+  for(unsigned int fieldnr=0; fieldnr < share->fields; fieldnr++)
+  {
+    drizzle::Table::Field pfield= table.field(fieldnr);
+
+    bool field_is_stored= true;
+
+    enum_field_types drizzle_field_type=
+      proto_field_type_to_drizzle_type(pfield.type());
+
+    if(drizzle_field_type==DRIZZLE_TYPE_VIRTUAL)
+    {
+      drizzle::Table::Field::VirtualFieldOptions field_options=
+	pfield.virtual_options();
+
+      field_is_stored= field_options.physically_stored();
+    }
+
+    if(!field_is_stored)
+    {
+      field_offsets[fieldnr]= stored_columns_reclength;
+      stored_columns_reclength+= field_pack_length[fieldnr];
+    }
   }
 
   ulong null_bits= null_fields;
@@ -657,7 +779,9 @@
     null_bits++;
   ulong data_offset= (null_bits + 7)/8;
 
+
   share->reclength+= data_offset;
+  share->stored_rec_length+= data_offset;
 
   ulong rec_buff_length;
 
@@ -670,8 +794,113 @@
                                      rec_buff_length)))
     abort();
 
+  memset(record, 0, rec_buff_length);
+
+  int null_count= 0;
+
+  if(!table_options.pack_record())
+  {
+    null_count++; // one bit for delete mark.
+    *record|= 1;
+  }
+
   share->default_values= record;
 
+  if(interval_count)
+  {
+    share->intervals= (TYPELIB*)alloc_root(&share->mem_root,
+					   interval_count*sizeof(TYPELIB));
+  }
+  else
+    share->intervals= NULL;
+
+  share->fieldnames.type_names= (const char**)alloc_root(&share->mem_root,
+			          (share->fields+1)*sizeof(char*));
+
+  share->fieldnames.type_lengths= (unsigned int*) alloc_root(&share->mem_root,
+				  (share->fields+1)*sizeof(unsigned int));
+
+  share->fieldnames.type_names[share->fields]= NULL;
+  share->fieldnames.type_lengths[share->fields]= 0;
+  share->fieldnames.count= share->fields;
+
+
+  /* Now fix the TYPELIBs for the intervals (enum values)
+     and field names.
+   */
+
+  uint32_t interval_nr= 0;
+
+  for(unsigned int fieldnr=0; fieldnr < share->fields; fieldnr++)
+  {
+    drizzle::Table::Field pfield= table.field(fieldnr);
+
+    /* field names */
+    share->fieldnames.type_names[fieldnr]= strmake_root(&share->mem_root,
+							pfield.name().c_str(),
+							pfield.name().length());
+
+    share->fieldnames.type_lengths[fieldnr]= pfield.name().length();
+
+    /* enum typelibs */
+    if(pfield.type() != drizzle::Table::Field::ENUM)
+      continue;
+
+    drizzle::Table::Field::SetFieldOptions field_options=
+      pfield.set_options();
+
+    const CHARSET_INFO *charset= get_charset(field_options.has_collation_id()?
+					     field_options.collation_id() : 0);
+
+    if (!charset)
+      charset= default_charset_info;
+
+    TYPELIB *t= &(share->intervals[interval_nr]);
+
+    t->type_names= (const char**)alloc_root(&share->mem_root,
+			   (field_options.field_value_size()+1)*sizeof(char*));
+
+    t->type_lengths= (unsigned int*) alloc_root(&share->mem_root,
+		     (field_options.field_value_size()+1)*sizeof(unsigned int));
+
+    t->type_names[field_options.field_value_size()]= NULL;
+    t->type_lengths[field_options.field_value_size()]= 0;
+
+    t->count= field_options.field_value_size();
+    t->name= NULL;
+
+    for(int n=0; n < field_options.field_value_size(); n++)
+    {
+      t->type_names[n]= strmake_root(&share->mem_root,
+				     field_options.field_value(n).c_str(),
+				     field_options.field_value(n).length());
+
+      /* Go ask the charset what the length is as for "" length=1
+	 and there's stripping spaces or some other crack going on.
+       */
+      uint32_t lengthsp;
+      lengthsp= charset->cset->lengthsp(charset, t->type_names[n],
+					field_options.field_value(n).length());
+      t->type_lengths[n]= lengthsp;
+    }
+    interval_nr++;
+  }
+
+
+  /* and read the fields */
+  interval_nr= 0;
+
+  bool use_hash= share->fields >= MAX_FIELDS_BEFORE_HASH;
+
+  if(use_hash)
+    use_hash= !hash_init(&share->name_hash,
+			 system_charset_info,
+			 share->fields, 0, 0,
+			 (hash_get_key) get_field_name, 0, 0);
+
+  unsigned char* null_pos= record;;
+  int null_bit_pos= (table_options.pack_record()) ? 0 : 1;
+
   for(unsigned int fieldnr=0; fieldnr < share->fields; fieldnr++)
   {
     drizzle::Table::Field pfield= table.field(fieldnr);
@@ -690,7 +919,7 @@
       column_format= COLUMN_FORMAT_TYPE_DYNAMIC;
       break;
     default:
-      abort();
+      assert(1);
     }
 
     Field::utype unireg_type= Field::NONE;
@@ -715,7 +944,7 @@
 	unireg_type= Field::TIMESTAMP_DN_FIELD;
       }
       else
-	abort(); // Invalid update value.
+	assert(1); // Invalid update value.
     }
     else if (pfield.has_options()
 	     && pfield.options().has_update_value()
@@ -764,25 +993,314 @@
       share->vfields++;
     }
 
-    // TODO: charset (well... collation)
-
-/*    share->field[fieldnr]= make_field(share, record+recordpos,
-				      field_length,
-				      null_pos, null_bit_pos,
-				      pack_flag,
-				      field_type,
-				      charset,
-				      (Field::utype) MTYP_TYPENR(unireg_type),
-				      (interval_nr ?
-				       share->intervals+interval_nr-1 :
-				       (TYPELIB*) 0),
-				      share->fieldnames.type_names[i]);
-*/
-  }
+    const CHARSET_INFO *charset= &my_charset_bin;
+
+    if(field_type==DRIZZLE_TYPE_BLOB
+       || field_type==DRIZZLE_TYPE_VARCHAR)
+    {
+      drizzle::Table::Field::StringFieldOptions field_options=
+	pfield.string_options();
+
+      charset= get_charset(field_options.has_collation_id()?
+			   field_options.collation_id() : 0);
+
+      if (!charset)
+	charset= default_charset_info;
+
+    }
+
+    if(field_type==DRIZZLE_TYPE_ENUM)
+    {
+      drizzle::Table::Field::SetFieldOptions field_options=
+	pfield.set_options();
+
+      charset= get_charset(field_options.has_collation_id()?
+			   field_options.collation_id() : 0);
+
+      if (!charset)
+	charset= default_charset_info;
+
+    }
+
+    Item *default_value= NULL;
+
+    if(pfield.options().has_default_value()
+       || pfield.options().has_default_null()
+       || pfield.options().has_default_bin_value())
+    {
+      default_value= default_value_item(field_type,
+					charset,
+					pfield.options().default_null(),
+					pfield.options().default_value(),
+					pfield.options().default_bin_value());
+    }
+
+    uint32_t pack_flag= pfield.pack_flag(); /* TODO: MUST DIE */
+
+    Table temp_table; /* Use this so that BLOB DEFAULT '' works */
+    memset(&temp_table, 0, sizeof(temp_table));
+    temp_table.s= share;
+    temp_table.in_use= session;
+    temp_table.s->db_low_byte_first= 1; //handler->low_byte_first();
+    temp_table.s->blob_ptr_size= portable_sizeof_char_ptr;
+
+    Field* f= make_field(share, &share->mem_root,
+			 record+field_offsets[fieldnr]+data_offset,
+			 pfield.options().length(),
+			 null_pos,
+			 null_bit_pos,
+			 pack_flag,
+			 field_type,
+			 charset,
+			 (Field::utype) MTYP_TYPENR(unireg_type),
+			 ((field_type==DRIZZLE_TYPE_ENUM)?
+			 share->intervals+(interval_nr++)
+			 : (TYPELIB*) 0),
+			share->fieldnames.type_names[fieldnr]);
+
+    share->field[fieldnr]= f;
+
+    f->init(&temp_table); /* blob default values need table obj */
+
+    if(!(f->flags & NOT_NULL_FLAG))
+    {
+      *f->null_ptr|= f->null_bit;
+      if (!(null_bit_pos= (null_bit_pos + 1) & 7))
+	null_pos++;
+      null_count++;
+    }
+
+    if(default_value)
+    {
+      int res= default_value->save_in_field(f, 1);
+      (void)res; // TODO error handle;
+    }
+    else if(f->real_type() == DRIZZLE_TYPE_ENUM &&
+	    (f->flags & NOT_NULL_FLAG))
+    {
+      f->set_notnull();
+      f->store((int64_t) 1, true);
+    }
+    else
+      f->reset();
+
+    /* hack to undo f->init() */
+    f->table= NULL;
+    f->orig_table= NULL;
+
+    f->field_index= fieldnr;
+    f->comment= comment;
+    f->vcol_info= vcol_info;
+    f->is_stored= field_is_stored;
+    if(!default_value
+       && !(f->unireg_check==Field::NEXT_NUMBER)
+       && (f->flags & NOT_NULL_FLAG)
+       && (f->real_type() != DRIZZLE_TYPE_TIMESTAMP))
+      f->flags|= NO_DEFAULT_VALUE_FLAG;
+
+    if(f->unireg_check == Field::NEXT_NUMBER)
+      share->found_next_number_field= &(share->field[fieldnr]);
+
+    if(share->timestamp_field == f)
+      share->timestamp_field_offset= fieldnr;
+
+    if (use_hash) /* supposedly this never fails... but comments lie */
+      (void) my_hash_insert(&share->name_hash,
+			    (unsigned char*)&(share->field[fieldnr]));
+
+    if(!f->is_stored)
+    {
+      share->stored_fields--;
+    }
+  }
+
+  keyinfo= share->key_info;
+  for (unsigned int keynr=0; keynr < share->keys; keynr++, keyinfo++)
+  {
+    key_part= keyinfo->key_part;
+
+    for(unsigned int partnr= 0;
+	partnr < keyinfo->key_parts;
+	partnr++, key_part++)
+    {
+      /* Fix up key_part->offset by adding data_offset.
+	 We really should compute offset as well.
+	 But at least this way we are a little better. */
+      key_part->offset= field_offsets[key_part->fieldnr-1] + data_offset;
+    }
+  }
+
+  /*
+    We need to set the unused bits to 1. If the number of bits is a multiple
+    of 8 there are no unused bits.
+  */
+
+  if (null_count & 7)
+    *(record + null_count / 8)|= ~(((unsigned char) 1 << (null_count & 7)) - 1);
+
+  share->null_bytes= (null_pos - (unsigned char*) record +
+		      (null_bit_pos + 7) / 8);
+
+  share->last_null_bit_pos= null_bit_pos;
 
   free(field_offsets);
   free(field_pack_length);
 
+  handler *handler_file;
+
+  if(!(handler_file= get_new_handler(share, session->mem_root,
+				     share->db_type())))
+    abort(); // FIXME
+
+  /* Fix key stuff */
+  if (share->key_parts)
+  {
+    uint32_t primary_key=(uint32_t) (find_type((char*) "PRIMARY",
+				       &share->keynames, 3) - 1);
+
+    int64_t ha_option= handler_file->ha_table_flags();
+
+    keyinfo= share->key_info;
+    key_part= keyinfo->key_part;
+
+    for (uint32_t key=0 ; key < share->keys ; key++,keyinfo++)
+    {
+      uint32_t usable_parts= 0;
+
+      if (primary_key >= MAX_KEY && (keyinfo->flags & HA_NOSAME))
+      {
+	/*
+	  If the UNIQUE key doesn't have NULL columns and is not a part key
+	  declare this as a primary key.
+	*/
+	primary_key=key;
+	for (uint32_t i=0 ; i < keyinfo->key_parts ;i++)
+	{
+	  uint32_t fieldnr= key_part[i].fieldnr;
+	  if (!fieldnr ||
+	      share->field[fieldnr-1]->null_ptr ||
+	      share->field[fieldnr-1]->key_length() !=
+	      key_part[i].length)
+	  {
+	    primary_key=MAX_KEY;                // Can't be used
+	    break;
+	  }
+	}
+      }
+
+      for (uint32_t i=0 ; i < keyinfo->key_parts ; key_part++,i++)
+      {
+        Field *field;
+	if (!key_part->fieldnr)
+        {
+//          error= 4;                             // Wrong file
+	  abort(); // goto err;
+        }
+        field= key_part->field= share->field[key_part->fieldnr-1];
+        key_part->type= field->key_type();
+        if (field->null_ptr)
+        {
+          key_part->null_offset=(uint32_t) ((unsigned char*) field->null_ptr -
+                                        share->default_values);
+          key_part->null_bit= field->null_bit;
+          key_part->store_length+=HA_KEY_NULL_LENGTH;
+          keyinfo->flags|=HA_NULL_PART_KEY;
+          keyinfo->extra_length+= HA_KEY_NULL_LENGTH;
+          keyinfo->key_length+= HA_KEY_NULL_LENGTH;
+        }
+        if (field->type() == DRIZZLE_TYPE_BLOB ||
+            field->real_type() == DRIZZLE_TYPE_VARCHAR)
+        {
+          if (field->type() == DRIZZLE_TYPE_BLOB)
+            key_part->key_part_flag|= HA_BLOB_PART;
+          else
+            key_part->key_part_flag|= HA_VAR_LENGTH_PART;
+          keyinfo->extra_length+=HA_KEY_BLOB_LENGTH;
+          key_part->store_length+=HA_KEY_BLOB_LENGTH;
+          keyinfo->key_length+= HA_KEY_BLOB_LENGTH;
+        }
+        if (i == 0 && key != primary_key)
+          field->flags |= (((keyinfo->flags & HA_NOSAME) &&
+                           (keyinfo->key_parts == 1)) ?
+                           UNIQUE_KEY_FLAG : MULTIPLE_KEY_FLAG);
+        if (i == 0)
+          field->key_start.set_bit(key);
+        if (field->key_length() == key_part->length &&
+            !(field->flags & BLOB_FLAG))
+        {
+          if (handler_file->index_flags(key, i, 0) & HA_KEYREAD_ONLY)
+          {
+            share->keys_for_keyread.set_bit(key);
+            field->part_of_key.set_bit(key);
+            field->part_of_key_not_clustered.set_bit(key);
+          }
+          if (handler_file->index_flags(key, i, 1) & HA_READ_ORDER)
+            field->part_of_sortkey.set_bit(key);
+        }
+        if (!(key_part->key_part_flag & HA_REVERSE_SORT) &&
+            usable_parts == i)
+          usable_parts++;			// For FILESORT
+        field->flags|= PART_KEY_FLAG;
+        if (key == primary_key)
+        {
+          field->flags|= PRI_KEY_FLAG;
+          /*
+            If this field is part of the primary key and all keys contains
+            the primary key, then we can use any key to find this column
+          */
+          if (ha_option & HA_PRIMARY_KEY_IN_READ_INDEX)
+          {
+            field->part_of_key= share->keys_in_use;
+            if (field->part_of_sortkey.is_set(key))
+              field->part_of_sortkey= share->keys_in_use;
+          }
+        }
+        if (field->key_length() != key_part->length)
+        {
+          key_part->key_part_flag|= HA_PART_KEY_SEG;
+        }
+      }
+      keyinfo->usable_key_parts= usable_parts; // Filesort
+
+      set_if_bigger(share->max_key_length,keyinfo->key_length+
+                    keyinfo->key_parts);
+      share->total_key_length+= keyinfo->key_length;
+      /*
+        MERGE tables do not have unique indexes. But every key could be
+        an unique index on the underlying MyISAM table. (Bug #10400)
+      */
+      if ((keyinfo->flags & HA_NOSAME) ||
+          (ha_option & HA_ANY_INDEX_MAY_BE_UNIQUE))
+        set_if_bigger(share->max_unique_length,keyinfo->key_length);
+    }
+    if (primary_key < MAX_KEY &&
+	(share->keys_in_use.is_set(primary_key)))
+    {
+      share->primary_key= primary_key;
+      /*
+	If we are using an integer as the primary key then allow the user to
+	refer to it as '_rowid'
+      */
+      if (share->key_info[primary_key].key_parts == 1)
+      {
+	Field *field= share->key_info[primary_key].key_part[0].field;
+	if (field && field->result_type() == INT_RESULT)
+        {
+          /* note that fieldnr here (and rowid_field_offset) starts from 1 */
+	  share->rowid_field_offset= (share->key_info[primary_key].key_part[0].
+                                      fieldnr);
+        }
+      }
+
+    }
+    else
+      share->primary_key = MAX_KEY; // we do not have a primary key
+  }
+  else
+    share->primary_key= MAX_KEY;
+
+  delete handler_file;
+
   return 0;
 }
 
@@ -970,14 +1488,13 @@
   uint32_t new_frm_ver, field_pack_length, new_field_pack_flag;
   uint32_t interval_count, interval_parts, read_length, int_length;
   uint32_t db_create_options, keys, key_parts, n_length;
-  uint32_t key_info_length, com_length, null_bit_pos=0;
+  uint32_t key_info_length, com_length;
   uint32_t vcol_screen_length;
   uint32_t i,j;
-  bool use_hash;
   unsigned char forminfo[288];
   char *names, *comment_pos, *vcol_screen_pos;
   unsigned char *record;
-  unsigned char *disk_buff, *strpos, *null_flags=NULL, *null_pos=NULL;
+  unsigned char *disk_buff, *strpos;
   ulong pos, record_offset, rec_buff_length;
   handler *handler_file= 0;
   KEY	*keyinfo;
@@ -1037,8 +1554,6 @@
 
     for (j=keyinfo->key_parts ; j-- ; key_part++)
     {
-      key_part->offset= (uint32_t) uint2korr(strpos+2)-1;
-      key_part->key_type=	(uint32_t) uint2korr(strpos+5);
       if (new_frm_ver >= 1)
       {
 	strpos+=9;
@@ -1068,7 +1583,6 @@
   }
 
   assert(share->reclength == uint2korr((head+16)));
-  share->stored_rec_length= share->reclength;
 
   record_offset= (ulong) (uint2korr(head+6)+
                           ((uint2korr(head+14) == 0xffff ?
@@ -1152,14 +1666,16 @@
   error=4;
   /* head+59 was extra_rec_buf_length */
   rec_buff_length= ALIGN_SIZE(share->reclength + 1);
-  share->rec_buff_length= rec_buff_length;
+
   if (!(record= (unsigned char *) alloc_root(&share->mem_root,
                                      rec_buff_length)))
     goto err;                                   /* purecov: inspected */
-  share->default_values= record;
+
   if (pread(file, record, (size_t) share->reclength, record_offset) == 0)
     goto err;                                   /* purecov: inspected */
 
+  assert(memcmp(share->default_values, record, share->reclength)==0);
+
   lseek(file,pos+288,SEEK_SET);
 
 //  share->fields= uint2korr(forminfo+258);
@@ -1186,7 +1702,7 @@
 			       vcol_screen_length)))))
     goto err;                                   /* purecov: inspected */
 
-  share->field= field_ptr;
+//  share->field= field_ptr;
   read_length=(uint32_t) (share->fields * field_pack_length +
 		      pos+ (uint32_t) (n_length+int_length+com_length+
 		                   vcol_screen_length));
@@ -1194,11 +1710,10 @@
     goto err;                                   /* purecov: inspected */
   strpos= disk_buff+pos;
 
-  share->intervals= (TYPELIB*) (field_ptr+share->fields+1);
-  interval_array= (const char **) (share->intervals+interval_count);
+//  share->intervals= (TYPELIB*) (field_ptr+share->fields+1);
+  interval_array= (const char **) ((field_ptr+share->fields+1)+interval_count);
   names= (char*) (interval_array+share->fields+interval_parts+keys+3);
-  if (!interval_count)
-    share->intervals= 0;			// For better debugging
+
   memcpy(names, strpos+(share->fields*field_pack_length),
 	 (uint32_t) (n_length+int_length));
   comment_pos= (char *)(disk_buff+read_length-com_length-vcol_screen_length);
@@ -1206,31 +1721,8 @@
   memcpy(vcol_screen_pos, disk_buff+read_length-vcol_screen_length,
          vcol_screen_length);
 
-  fix_type_pointers(&interval_array, &share->fieldnames, 1, &names);
-  if (share->fieldnames.count != share->fields)
-    goto err;
-  fix_type_pointers(&interval_array, share->intervals, interval_count,
-		    &names);
-
-  {
-    /* Set ENUM and SET lengths */
-    TYPELIB *interval;
-    for (interval= share->intervals;
-         interval < share->intervals + interval_count;
-         interval++)
-    {
-      uint32_t count= (uint32_t) (interval->count + 1) * sizeof(uint32_t);
-      if (!(interval->type_lengths= (uint32_t *) alloc_root(&share->mem_root,
-                                                        count)))
-        goto err;
-      for (count= 0; count < interval->count; count++)
-      {
-        char *val= (char*) interval->type_names[count];
-        interval->type_lengths[count]= strlen(val);
-      }
-      interval->type_lengths[count]= 0;
-    }
-  }
+//  fix_type_pointers(&interval_array, &share->fieldnames, 1, &names);
+  assert(share->fieldnames.count == share->fields);
 
 /*  if (keynames)
     fix_type_pointers(&interval_array, &share->keynames, 1, &keynames);*/
@@ -1242,325 +1734,14 @@
 
   record= share->default_values-1;              /* Fieldstart = 1 */
 
-  null_flags= null_pos= (unsigned char*) record+1;
-  null_bit_pos= (db_create_options & HA_OPTION_PACK_RECORD) ? 0 : 1;
-  /*
-    null_bytes below is only correct under the condition that
-    there are no bit fields.  Correct values is set below after the
-    table struct is initialized
-  */
-  share->null_bytes= (share->null_fields + null_bit_pos + 7) / 8;
-
-  use_hash= share->fields >= MAX_FIELDS_BEFORE_HASH;
-  if (use_hash)
-    use_hash= !hash_init(&share->name_hash,
-			 system_charset_info,
-			 share->fields,0,0,
-			 (hash_get_key) get_field_name,0,0);
-
-  for (i=0 ; i < share->fields; i++, strpos+=field_pack_length, field_ptr++)
-  {
-    uint32_t pack_flag, interval_nr, unireg_type, recpos, field_length;
-    uint32_t vcol_info_length=0;
-    enum_field_types field_type;
-    enum column_format_type column_format= COLUMN_FORMAT_TYPE_DEFAULT;
-    const CHARSET_INFO *charset= NULL;
-    LEX_STRING comment;
-    virtual_column_info *vcol_info= NULL;
-    bool fld_is_stored= true;
-
-    if (field_extra_info)
-    {
-      char tmp= field_extra_info[i];
-      column_format= (enum column_format_type)
-                    ((tmp >> COLUMN_FORMAT_SHIFT) & COLUMN_FORMAT_MASK);
-    }
-    if (new_frm_ver >= 3)
-    {
-      /* new frm file in 4.1 */
-      field_length= uint2korr(strpos+3);
-      recpos=	    uint3korr(strpos+5);
-      pack_flag=    uint2korr(strpos+8);
-      unireg_type=  (uint32_t) strpos[10];
-      interval_nr=  (uint32_t) strpos[12];
-      uint32_t comment_length=uint2korr(strpos+15);
-      field_type=(enum_field_types) (uint32_t) strpos[13];
-
-      {
-        if (!strpos[14])
-          charset= &my_charset_bin;
-        else if (!(charset=get_charset((uint32_t) strpos[14])))
-        {
-          error= 5; // Unknown or unavailable charset
-          errarg= (int) strpos[14];
-          goto err;
-        }
-      }
-      if (field_type == DRIZZLE_TYPE_VIRTUAL)
-      {
-        assert(interval_nr); // Expect non-null expression
-        /*
-          The interval_id byte in the .frm file stores the length of the
-          expression statement for a virtual column.
-        */
-        vcol_info_length= interval_nr;
-        interval_nr= 0;
-      }
-      if (!comment_length)
-      {
-	comment.str= (char*) "";
-	comment.length=0;
-      }
-      else
-      {
-	comment.str=    strmake_root(&share->mem_root, comment_pos, comment_length);
-	comment.length= comment_length;
-	comment_pos+=   comment_length;
-      }
-      if (vcol_info_length)
-      {
-        /*
-          Get virtual column data stored in the .frm file as follows:
-          byte 1      = 1 (always 1 to allow for future extensions)
-          byte 2      = sql_type
-          byte 3      = flags (as of now, 0 - no flags, 1 - field is physically stored)
-          byte 4-...  = virtual column expression (text data)
-        */
-        vcol_info= new virtual_column_info();
-        if ((uint32_t)vcol_screen_pos[0] != 1)
-        {
-          error= 4;
-          goto err;
-        }
-        field_type= (enum_field_types) (unsigned char) vcol_screen_pos[1];
-        fld_is_stored= (bool) (uint32_t) vcol_screen_pos[2];
-        vcol_info->expr_str.str= (char *)memdup_root(&share->mem_root,
-                                                     vcol_screen_pos+
-                                                       (uint32_t)FRM_VCOL_HEADER_SIZE,
-                                                     vcol_info_length-
-                                                       (uint32_t)FRM_VCOL_HEADER_SIZE);
-        vcol_info->expr_str.length= vcol_info_length-(uint32_t)FRM_VCOL_HEADER_SIZE;
-        vcol_screen_pos+= vcol_info_length;
-        share->vfields++;
-      }
-    }
-    else
-    {
-      assert(false); /* Old (pre-4.1) FRM file. This is Drizzle. */
-    }
-
-    if (interval_nr && charset->mbminlen > 1)
-    {
-      /* Unescape UCS2 intervals from HEX notation */
-      TYPELIB *interval= share->intervals + interval_nr - 1;
-      unhex_type2(interval);
-    }
-
-    *field_ptr= reg_field=
-      make_field(share, record+recpos,
-		 (uint32_t) field_length,
-		 null_pos, null_bit_pos,
-		 pack_flag,
-		 field_type,
-		 charset,
-		 (Field::utype) MTYP_TYPENR(unireg_type),
-		 (interval_nr ?
-		  share->intervals+interval_nr-1 :
-		  (TYPELIB*) 0),
-		 share->fieldnames.type_names[i]);
-    if (!reg_field)				// Not supported field type
-    {
-      error= 4;
-      goto err;			/* purecov: inspected */
-    }
-
-    reg_field->flags|= ((uint32_t)column_format << COLUMN_FORMAT_FLAGS);
-    reg_field->field_index= i;
-    reg_field->comment=comment;
-    reg_field->vcol_info= vcol_info;
-    reg_field->is_stored= fld_is_stored;
-    if (!(reg_field->flags & NOT_NULL_FLAG))
-    {
-      if (!(null_bit_pos= (null_bit_pos + 1) & 7))
-        null_pos++;
-    }
-    if (f_no_default(pack_flag))
-      reg_field->flags|= NO_DEFAULT_VALUE_FLAG;
-
-    if (reg_field->unireg_check == Field::NEXT_NUMBER)
-      share->found_next_number_field= field_ptr;
-    if (share->timestamp_field == reg_field)
-      share->timestamp_field_offset= i;
-
-    if (use_hash)
-      (void) my_hash_insert(&share->name_hash,
-                            (unsigned char*) field_ptr); // never fail
-    if (!reg_field->is_stored)
-    {
-      share->stored_fields--;
-      if (share->stored_rec_length>=recpos)
-        share->stored_rec_length= recpos-1;
-    }
-  }
-  *field_ptr=0;					// End marker
   /* Sanity checks: */
   assert(share->fields>=share->stored_fields);
   assert(share->reclength>=share->stored_rec_length);
 
-  /* Fix key->name and key_part->field */
-  if (key_parts)
-  {
-    uint32_t primary_key=(uint32_t) (find_type((char*) "PRIMARY",
-				       &share->keynames, 3) - 1);
-    int64_t ha_option= handler_file->ha_table_flags();
-    keyinfo= share->key_info;
-    key_part= keyinfo->key_part;
-
-    for (uint32_t key=0 ; key < share->keys ; key++,keyinfo++)
-    {
-      uint32_t usable_parts= 0;
-//      keyinfo->name=(char*) share->keynames.type_names[key];
-
-      if (primary_key >= MAX_KEY && (keyinfo->flags & HA_NOSAME))
-      {
-	/*
-	  If the UNIQUE key doesn't have NULL columns and is not a part key
-	  declare this as a primary key.
-	*/
-	primary_key=key;
-	for (i=0 ; i < keyinfo->key_parts ;i++)
-	{
-	  uint32_t fieldnr= key_part[i].fieldnr;
-	  if (!fieldnr ||
-	      share->field[fieldnr-1]->null_ptr ||
-	      share->field[fieldnr-1]->key_length() !=
-	      key_part[i].length)
-	  {
-	    primary_key=MAX_KEY;                // Can't be used
-	    break;
-	  }
-	}
-      }
-
-      for (i=0 ; i < keyinfo->key_parts ; key_part++,i++)
-      {
-        Field *field;
-	if (!key_part->fieldnr)
-        {
-          error= 4;                             // Wrong file
-          goto err;
-        }
-        field= key_part->field= share->field[key_part->fieldnr-1];
-        key_part->type= field->key_type();
-        if (field->null_ptr)
-        {
-          key_part->null_offset=(uint32_t) ((unsigned char*) field->null_ptr -
-                                        share->default_values);
-          key_part->null_bit= field->null_bit;
-          key_part->store_length+=HA_KEY_NULL_LENGTH;
-          keyinfo->flags|=HA_NULL_PART_KEY;
-          keyinfo->extra_length+= HA_KEY_NULL_LENGTH;
-          keyinfo->key_length+= HA_KEY_NULL_LENGTH;
-        }
-        if (field->type() == DRIZZLE_TYPE_BLOB ||
-            field->real_type() == DRIZZLE_TYPE_VARCHAR)
-        {
-          if (field->type() == DRIZZLE_TYPE_BLOB)
-            key_part->key_part_flag|= HA_BLOB_PART;
-          else
-            key_part->key_part_flag|= HA_VAR_LENGTH_PART;
-          keyinfo->extra_length+=HA_KEY_BLOB_LENGTH;
-          key_part->store_length+=HA_KEY_BLOB_LENGTH;
-          keyinfo->key_length+= HA_KEY_BLOB_LENGTH;
-        }
-        if (i == 0 && key != primary_key)
-          field->flags |= (((keyinfo->flags & HA_NOSAME) &&
-                           (keyinfo->key_parts == 1)) ?
-                           UNIQUE_KEY_FLAG : MULTIPLE_KEY_FLAG);
-        if (i == 0)
-          field->key_start.set_bit(key);
-        if (field->key_length() == key_part->length &&
-            !(field->flags & BLOB_FLAG))
-        {
-          if (handler_file->index_flags(key, i, 0) & HA_KEYREAD_ONLY)
-          {
-            share->keys_for_keyread.set_bit(key);
-            field->part_of_key.set_bit(key);
-            field->part_of_key_not_clustered.set_bit(key);
-          }
-          if (handler_file->index_flags(key, i, 1) & HA_READ_ORDER)
-            field->part_of_sortkey.set_bit(key);
-        }
-        if (!(key_part->key_part_flag & HA_REVERSE_SORT) &&
-            usable_parts == i)
-          usable_parts++;			// For FILESORT
-        field->flags|= PART_KEY_FLAG;
-        if (key == primary_key)
-        {
-          field->flags|= PRI_KEY_FLAG;
-          /*
-            If this field is part of the primary key and all keys contains
-            the primary key, then we can use any key to find this column
-          */
-          if (ha_option & HA_PRIMARY_KEY_IN_READ_INDEX)
-          {
-            field->part_of_key= share->keys_in_use;
-            if (field->part_of_sortkey.is_set(key))
-              field->part_of_sortkey= share->keys_in_use;
-          }
-        }
-        if (field->key_length() != key_part->length)
-        {
-          key_part->key_part_flag|= HA_PART_KEY_SEG;
-        }
-      }
-      keyinfo->usable_key_parts= usable_parts; // Filesort
-
-      set_if_bigger(share->max_key_length,keyinfo->key_length+
-                    keyinfo->key_parts);
-      share->total_key_length+= keyinfo->key_length;
-      /*
-        MERGE tables do not have unique indexes. But every key could be
-        an unique index on the underlying MyISAM table. (Bug #10400)
-      */
-      if ((keyinfo->flags & HA_NOSAME) ||
-          (ha_option & HA_ANY_INDEX_MAY_BE_UNIQUE))
-        set_if_bigger(share->max_unique_length,keyinfo->key_length);
-    }
-    if (primary_key < MAX_KEY &&
-	(share->keys_in_use.is_set(primary_key)))
-    {
-      share->primary_key= primary_key;
-      /*
-	If we are using an integer as the primary key then allow the user to
-	refer to it as '_rowid'
-      */
-      if (share->key_info[primary_key].key_parts == 1)
-      {
-	Field *field= share->key_info[primary_key].key_part[0].field;
-	if (field && field->result_type() == INT_RESULT)
-        {
-          /* note that fieldnr here (and rowid_field_offset) starts from 1 */
-	  share->rowid_field_offset= (share->key_info[primary_key].key_part[0].
-                                      fieldnr);
-        }
-      }
-    }
-    else
-      share->primary_key = MAX_KEY; // we do not have a primary key
-  }
-  else
-    share->primary_key= MAX_KEY;
+
   if (disk_buff)
     free(disk_buff);
   disk_buff= NULL;
-  if (new_field_pack_flag <= 1)
-  {
-    /* Old file format with default as not null */
-    uint32_t null_length= (share->null_fields+7)/8;
-    memset(share->default_values + (null_flags - (unsigned char*) record),
-          null_length, 255);
-  }
 
   if (share->found_next_number_field)
   {
@@ -1600,9 +1781,6 @@
     the correct null_bytes can now be set, since bitfields have been taken
     into account
   */
-  share->null_bytes= (null_pos - (unsigned char*) null_flags +
-                      (null_bit_pos + 7) / 8);
-  share->last_null_bit_pos= null_bit_pos;
 
   share->db_low_byte_first= handler_file->low_byte_first();
   share->column_bitmap_size= bitmap_buffer_size(share->fields);
@@ -1705,6 +1883,7 @@
   memcpy(db_name_string,
          table->s->normalized_path.str,
          table->s->normalized_path.length);
+  db_name_string[table->s->normalized_path.length]= '\0';
   dir_length= dirname_length(db_name_string);
   db_name_string[dir_length - 1]= 0;
   home_dir_length= dirname_length(db_name_string);
@@ -4762,7 +4941,7 @@
   List_iterator_fast<Create_field> it(field_list);
   while ((cdef= it++))
   {
-    *field= make_field(share, 0, cdef->length,
+    *field= make_field(share, NULL, 0, cdef->length,
                        (unsigned char*) (f_maybe_null(cdef->pack_flag) ? "" : 0),
                        f_maybe_null(cdef->pack_flag) ? 1 : 0,
                        cdef->pack_flag, cdef->sql_type, cdef->charset,

=== modified file 'drizzled/unireg.cc'
--- drizzled/unireg.cc	2009-02-21 03:33:30 +0000
+++ drizzled/unireg.cc	2009-02-26 20:57:39 +0000
@@ -102,10 +102,12 @@
 
   if (!table->ParseFromZeroCopyStream(input))
   {
+    delete input;
     close(fd);
     return -1;
   }
 
+  delete input;
   close(fd);
   return 0;
 }
@@ -408,6 +410,8 @@
     attribute= table_proto->add_field();
     attribute->set_name(field_arg->field_name);
 
+    attribute->set_pack_flag(field_arg->pack_flag); /* TODO: MUST DIE */
+
     if(f_maybe_null(field_arg->pack_flag))
     {
       drizzle::Table::Field::FieldConstraints *constraints;
@@ -483,13 +487,25 @@
           set_field_options->add_field_value(src);
         }
         set_field_options->set_count_elements(set_field_options->field_value_size());
+	set_field_options->set_collation_id(field_arg->charset->number);
+        set_field_options->set_collation(field_arg->charset->name);
         break;
       }
     case DRIZZLE_TYPE_BLOB:
-      attribute->set_type(drizzle::Table::Field::BLOB);
+      {
+	attribute->set_type(drizzle::Table::Field::BLOB);
+
+        drizzle::Table::Field::StringFieldOptions *string_field_options;
+
+        string_field_options= attribute->mutable_string_options();
+        string_field_options->set_collation_id(field_arg->charset->number);
+        string_field_options->set_collation(field_arg->charset->name);
+      }
+
       break;
     default:
-      abort();
+      assert(0); /* Tell us, since this shouldn't happend */
+      return;
     }
 
     if(field_arg->vcol_info)
@@ -501,7 +517,10 @@
       field_options->set_type(attribute->type());
       attribute->set_type(drizzle::Table::Field::VIRTUAL);
 
-      field_options->set_expression(field_arg->vcol_info->expr_str.str);
+      string expr(field_arg->vcol_info->expr_str.str,
+		  field_arg->vcol_info->expr_str.length);
+
+      field_options->set_expression(expr);
       field_options->set_physically_stored(field_arg->is_stored);
     }
 
@@ -524,7 +543,8 @@
       attribute->set_format(drizzle::Table::Field::DynamicFormat);
       break;
     default:
-      abort();
+      assert(0); /* Tell us, since this shouldn't happend */
+      return;
     }
 
     if (field_arg->comment.length)
@@ -553,6 +573,44 @@
       field_options->set_update_value("NOW()");
     }
 
+    if(field_arg->def)
+    {
+      drizzle::Table::Field::FieldOptions *field_options;
+      field_options= attribute->mutable_options();
+
+      if(field_arg->def->is_null())
+      {
+	field_options->set_default_null(true);
+      }
+      else
+      {
+	String d;
+	String *default_value= field_arg->def->val_str(&d);
+
+	if((field_arg->sql_type==DRIZZLE_TYPE_VARCHAR
+	    && field_arg->charset==&my_charset_bin)
+	   || (field_arg->sql_type==DRIZZLE_TYPE_BLOB
+	    && field_arg->charset==&my_charset_bin))
+	{
+	  string bin_default;
+	  bin_default.assign(default_value->c_ptr(),
+			     default_value->length());
+	  field_options->set_default_bin_value(bin_default);
+	}
+	else
+	{
+	  field_options->set_default_value(default_value->c_ptr());
+	}
+      }
+    }
+
+    {
+      drizzle::Table::Field::FieldOptions *field_options;
+      field_options= attribute->mutable_options();
+
+      field_options->set_length(field_arg->length);
+    }
+
     assert(field_arg->unireg_check == Field::NONE
 	   || field_arg->unireg_check == Field::NEXT_NUMBER
 	   || field_arg->unireg_check == Field::TIMESTAMP_DN_FIELD
@@ -742,9 +800,12 @@
 
       idxpart= idx->add_index_part();
 
-      idxpart->set_fieldnr(key_info[i].key_part[j].fieldnr+1);
+      idxpart->set_fieldnr(key_info[i].key_part[j].fieldnr);
 
       idxpart->set_compare_length(key_info[i].key_part[j].length);
+
+      idxpart->set_key_type(key_info[i].key_part[j].key_type);
+
     }
   }
 }
@@ -1461,7 +1522,7 @@
     /*
       regfield don't have to be deleted as it's allocated with sql_alloc()
     */
-    Field *regfield= make_field(&share,
+    Field *regfield= make_field(&share, NULL,
                                 buff+field->offset + data_offset,
                                 field->length,
                                 null_pos + null_count / 8,

=== modified file 'tests/r/alter_table.result'
--- tests/r/alter_table.result	2009-02-17 10:00:38 +0000
+++ tests/r/alter_table.result	2009-02-25 16:45:57 +0000
@@ -393,7 +393,7 @@
 alter table t1 modify a varchar(10) not null;
 drop table if exists t1;
 create table t1 (a int, b int, c int, d int, e int, f int, g int, h int,i int, primary key (a,b,c,d,e,f,g,i,h)) engine=MyISAM;
-insert into t1 (a) values(1);
+insert into t1 (a,b,c,d,e,f,g,h,i) values(1,1,1,1,1,1,1,1,1);
 show table status like 't1';
 Name	Engine	Version	Row_format	Rows	Avg_row_length	Data_length	Max_data_length	Index_length	Data_free	Auto_increment	Create_time	Update_time	Check_time	Collation	Checksum	Create_options	Comment
 t1	MyISAM	X	Fixed	1	37	X	X	X	X	X	X	X	X	utf8_general_ci	NULL		

=== modified file 'tests/r/bigint.result'
--- tests/r/bigint.result	2009-02-17 10:00:38 +0000
+++ tests/r/bigint.result	2009-02-25 16:45:57 +0000
@@ -168,14 +168,14 @@
 show create table t1;
 Table	Create Table
 t1	CREATE TABLE `t1` (
-  `a` int NOT NULL DEFAULT '0'
+  `a` int NOT NULL
 ) ENGINE=InnoDB
 drop table t1;
 create table t1 select 9223372036854775809 as 'a';
 show create table t1;
 Table	Create Table
 t1	CREATE TABLE `t1` (
-  `a` bigint NOT NULL DEFAULT '0'
+  `a` bigint NOT NULL
 ) ENGINE=InnoDB
 select * from t1;
 a
@@ -369,10 +369,10 @@
 create table t1 select -9223372036854775808 bi;
 describe t1;
 Field	Type	Null	Key	Default	Extra
-bi	bigint	NO		0	
+bi	bigint	NO		NULL	
 drop table t1;
 create table t1 select -9223372036854775809 bi;
 describe t1;
 Field	Type	Null	Key	Default	Extra
-bi	decimal(19,0)	NO		0	
+bi	decimal(19,0)	NO		NULL	
 drop table t1;

=== modified file 'tests/r/case.result'
--- tests/r/case.result	2009-02-17 10:00:38 +0000
+++ tests/r/case.result	2009-02-25 16:45:57 +0000
@@ -108,16 +108,16 @@
 t1	CREATE TABLE `t1` (
   `c1` varchar(1) CHARACTER SET utf8 COLLATE utf8_bin DEFAULT NULL,
   `c2` varchar(1) CHARACTER SET utf8 COLLATE utf8_bin DEFAULT NULL,
-  `c3` varbinary(4) NOT NULL DEFAULT '',
-  `c4` varbinary(4) NOT NULL DEFAULT '',
-  `c5` varbinary(4) NOT NULL DEFAULT '',
-  `c6` varbinary(4) NOT NULL DEFAULT '',
-  `c7` decimal(2,1) NOT NULL DEFAULT '0.0',
-  `c8` decimal(2,1) NOT NULL DEFAULT '0.0',
+  `c3` varbinary(4) NOT NULL,
+  `c4` varbinary(4) NOT NULL,
+  `c5` varbinary(4) NOT NULL,
+  `c6` varbinary(4) NOT NULL,
+  `c7` decimal(2,1) NOT NULL,
+  `c8` decimal(2,1) NOT NULL,
   `c9` decimal(2,1) DEFAULT NULL,
-  `c10` double NOT NULL DEFAULT '0',
-  `c11` double NOT NULL DEFAULT '0',
-  `c12` varbinary(5) NOT NULL DEFAULT ''
+  `c10` double NOT NULL,
+  `c11` double NOT NULL,
+  `c12` varbinary(5) NOT NULL
 ) ENGINE=InnoDB
 DROP TABLE t1;
 SELECT CASE 
@@ -156,12 +156,12 @@
 SHOW CREATE TABLE t1;
 Table	Create Table
 t1	CREATE TABLE `t1` (
-  `COALESCE(1)` int NOT NULL DEFAULT '0',
-  `COALESCE(1.0)` decimal(2,1) NOT NULL DEFAULT '0.0',
-  `COALESCE('a')` varchar(1) NOT NULL DEFAULT '',
-  `COALESCE(1,1.0)` decimal(2,1) NOT NULL DEFAULT '0.0',
-  `COALESCE(1,'1')` varbinary(4) NOT NULL DEFAULT '',
-  `COALESCE(1.1,'1')` varbinary(4) NOT NULL DEFAULT '',
+  `COALESCE(1)` int NOT NULL,
+  `COALESCE(1.0)` decimal(2,1) NOT NULL,
+  `COALESCE('a')` varchar(1) NOT NULL,
+  `COALESCE(1,1.0)` decimal(2,1) NOT NULL,
+  `COALESCE(1,'1')` varbinary(4) NOT NULL,
+  `COALESCE(1.1,'1')` varbinary(4) NOT NULL,
   `COALESCE('a' COLLATE utf8_bin,'b')` varchar(1) CHARACTER SET utf8 COLLATE utf8_bin DEFAULT NULL
 ) ENGINE=InnoDB
 DROP TABLE t1;

=== modified file 'tests/r/create.result'
--- tests/r/create.result	2009-02-17 10:00:38 +0000
+++ tests/r/create.result	2009-02-25 16:45:57 +0000
@@ -118,10 +118,10 @@
 Field	Type	Null	Key	Default	Extra
 a	datetime	YES		NULL	
 b	time	YES		NULL	
-c	date	NO		0000-00-00	
-d	int	NO		0	
-e	decimal(3,1)	NO		0.0	
-f	bigint	NO		0	
+c	date	NO		NULL	
+d	int	NO		NULL	
+e	decimal(3,1)	NO		NULL	
+f	bigint	NO		NULL	
 drop table t2;
 create table t2 select CAST("2001-12-29" AS DATE) as d, CAST("20:45:11" AS TIME) as t, CAST("2001-12-29  20:45:11" AS DATETIME) as dt;
 describe t2;
@@ -423,8 +423,8 @@
 explain t2;
 Field	Type	Null	Key	Default	Extra
 a	int	YES		NULL	
-b	bigint	NO		0	
-c	bigint	NO		0	
+b	bigint	NO		NULL	
+c	bigint	NO		NULL	
 d	date	YES		NULL	
 e	varchar(1)	YES		NULL	
 f	datetime	YES		NULL	
@@ -595,7 +595,7 @@
 t1	CREATE TABLE `t1` (
   `b` int DEFAULT NULL,
   `a` varchar(12) CHARACTER SET utf8 COLLATE utf8_bin NOT NULL,
-  `c` int NOT NULL DEFAULT '0',
+  `c` int NOT NULL,
   PRIMARY KEY (`a`)
 ) ENGINE=InnoDB
 drop table t1;
@@ -618,7 +618,7 @@
 show create table t1;
 Table	Create Table
 t1	CREATE TABLE `t1` (
-  `a` varchar(12) CHARACTER SET utf8 COLLATE utf8_bin NOT NULL DEFAULT '',
+  `a` varchar(12) CHARACTER SET utf8 COLLATE utf8_bin NOT NULL,
   `b` int NOT NULL,
   PRIMARY KEY (`a`)
 ) ENGINE=InnoDB
@@ -1592,6 +1592,8 @@
 
 
 CREATE TABLE t2(c1 TIMESTAMP, c2 TIMESTAMP DEFAULT 0);
+Warnings:
+Warning	1292	Incorrect datetime value: '0' for column 'c2' at row 1
 drop table t2;
 
 CREATE TABLE t2(c1 TIMESTAMP, c2 TIMESTAMP);

=== modified file 'tests/r/distinct.result'
--- tests/r/distinct.result	2009-02-11 02:30:38 +0000
+++ tests/r/distinct.result	2009-02-25 16:45:57 +0000
@@ -645,7 +645,7 @@
 1	1	3
 DROP TABLE t1;
 End of 5.0 tests
-CREATE TABLE t1(a INT, b INT, c INT, d INT, e INT,
+CREATE TABLE t1(a INT, b INT, c INT, d INT DEFAULT 0, e INT DEFAULT 0,
 PRIMARY KEY(a,b,c,d,e),
 KEY(a,b,d,c)
 );

=== modified file 'tests/r/func_math.result'
--- tests/r/func_math.result	2009-02-17 10:00:38 +0000
+++ tests/r/func_math.result	2009-02-25 16:45:57 +0000
@@ -174,7 +174,7 @@
 show create table t1;
 Table	Create Table
 t1	CREATE TABLE `t1` (
-  `round(1, 6)` int NOT NULL DEFAULT '0'
+  `round(1, 6)` int NOT NULL
 ) ENGINE=X
 select * from t1;
 round(1, 6)

=== modified file 'tests/r/func_time.result'
--- tests/r/func_time.result	2009-02-18 19:27:32 +0000
+++ tests/r/func_time.result	2009-02-25 16:45:57 +0000
@@ -674,7 +674,7 @@
 from_days(to_days("960101")) as b;
 describe t1;
 Field	Type	Null	Key	Default	Extra
-a	date	NO		0000-00-00	
+a	date	NO		NULL	
 b	date	YES		NULL	
 select * from t1;
 a	b

=== modified file 'tests/r/key.result'
--- tests/r/key.result	2009-02-17 10:00:38 +0000
+++ tests/r/key.result	2009-02-25 16:45:57 +0000
@@ -397,8 +397,8 @@
 Table	Create Table
 t1	CREATE TABLE `t1` (
   `c1` int DEFAULT NULL,
-  `c2` varchar(12) NOT NULL DEFAULT '',
-  `c3` varchar(123) NOT NULL DEFAULT '',
+  `c2` varchar(12) NOT NULL,
+  `c3` varchar(123) NOT NULL,
   `c4` timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP,
   PRIMARY KEY (`c2`,`c3`),
   UNIQUE KEY `i4` (`c4`),
@@ -429,9 +429,9 @@
 show create table t1;
 Table	Create Table
 t1	CREATE TABLE `t1` (
-  `c1` int NOT NULL DEFAULT '0',
-  `c2` varchar(12) NOT NULL DEFAULT '',
-  `c3` varchar(123) NOT NULL DEFAULT '',
+  `c1` int NOT NULL,
+  `c2` varchar(12) NOT NULL,
+  `c3` varchar(123) NOT NULL,
   `c4` timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP,
   KEY `i1` (`c1`),
   KEY `i5` (`c1`,`c2`,`c3`,`c4`),

=== modified file 'tests/r/null.result'
--- tests/r/null.result	2009-02-17 10:00:38 +0000
+++ tests/r/null.result	2009-02-25 16:45:57 +0000
@@ -184,7 +184,7 @@
   `c00` varbinary(0) DEFAULT NULL,
   `c01` varchar(6) DEFAULT NULL,
   `c02` varchar(6) DEFAULT NULL,
-  `c03` varchar(6) NOT NULL DEFAULT '',
+  `c03` varchar(6) NOT NULL,
   `c04` varchar(6) DEFAULT NULL,
   `c05` varchar(6) DEFAULT NULL,
   `c06` varchar(6) DEFAULT NULL,

=== modified file 'tests/r/parser.result'
--- tests/r/parser.result	2009-02-17 10:00:38 +0000
+++ tests/r/parser.result	2009-02-25 16:45:57 +0000
@@ -250,7 +250,7 @@
 CREATE TABLE table_25930_b SELECT "blah" - 1 FROM table_25930_a;
 desc table_25930_b;
 Field	Type	Null	Key	Default	Extra
-"blah" - 1	double	NO		0	
+"blah" - 1	double	NO		NULL	
 DROP TABLE table_25930_a;
 DROP TABLE table_25930_b;
 select pi(3.14);

=== modified file 'tests/r/select.result'
--- tests/r/select.result	2009-02-17 10:00:38 +0000
+++ tests/r/select.result	2009-02-25 16:45:57 +0000
@@ -3246,9 +3246,9 @@
 show create table t1;
 Table	Create Table
 t1	CREATE TABLE `t1` (
-  `i` decimal(19,0) NOT NULL DEFAULT '0',
-  `c` decimal(19,0) NOT NULL DEFAULT '0',
-  `co` decimal(19,0) NOT NULL DEFAULT '0'
+  `i` decimal(19,0) NOT NULL,
+  `c` decimal(19,0) NOT NULL,
+  `co` decimal(19,0) NOT NULL
 ) ENGINE=InnoDB
 drop table t1;
 select 

=== modified file 'tests/r/strict.result'
--- tests/r/strict.result	2009-02-17 10:00:38 +0000
+++ tests/r/strict.result	2009-02-25 16:45:57 +0000
@@ -438,7 +438,7 @@
 show create table t1;
 Table	Create Table
 t1	CREATE TABLE `t1` (
-  `a` int NOT NULL DEFAULT '0',
+  `a` int NOT NULL,
   `b` timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP,
   PRIMARY KEY (`a`)
 ) ENGINE=InnoDB
@@ -448,7 +448,7 @@
 show create table t1;
 Table	Create Table
 t1	CREATE TABLE `t1` (
-  `a` int NOT NULL DEFAULT '0',
+  `a` int NOT NULL,
   `b` timestamp NOT NULL DEFAULT '2005-01-02 03:04:05',
   PRIMARY KEY (`a`)
 ) ENGINE=InnoDB

=== modified file 'tests/r/subselect.result'
--- tests/r/subselect.result	2009-02-17 10:00:38 +0000
+++ tests/r/subselect.result	2009-02-25 16:45:57 +0000
@@ -921,7 +921,7 @@
 CREATE TABLE `t1` (
 `mot` varchar(30) NOT NULL default '',
 `topic` bigint NOT NULL default '0',
-`date` date NULL,
+`date` date NULL DEFAULT '2009-01-20',
 `pseudo` varchar(35) NOT NULL default '',
 PRIMARY KEY  (`mot`,`pseudo`,`date`,`topic`),
 KEY `pseudo` (`pseudo`,`date`,`topic`),
@@ -930,7 +930,7 @@
 CREATE TABLE `t2` (
 `mot` varchar(30) NOT NULL default '',
 `topic` bigint NOT NULL default '0',
-`date` date NULL,
+`date` date NULL default '1997-08-29',
 `pseudo` varchar(35) NOT NULL default '',
 PRIMARY KEY  (`mot`,`pseudo`,`date`,`topic`),
 KEY `pseudo` (`pseudo`,`date`,`topic`),
@@ -951,13 +951,13 @@
 2
 select * from t1;
 mot	topic	date	pseudo
-joce	1	0000-00-00	joce
-test	2	0000-00-00	test
+joce	1	2009-01-20	joce
+test	2	2009-01-20	test
 DELETE FROM t1 WHERE topic IN (SELECT DISTINCT topic FROM t2 WHERE NOT
 EXISTS(SELECT * FROM t3 WHERE numeropost=topic));
 select * from t1;
 mot	topic	date	pseudo
-joce	1	0000-00-00	joce
+joce	1	2009-01-20	joce
 drop table t1, t2, t3;
 SELECT * FROM (SELECT 1 as a,(SELECT a)) a;
 a	(SELECT a)
@@ -993,7 +993,7 @@
 SHOW CREATE TABLE t1;
 Table	Create Table
 t1	CREATE TABLE `t1` (
-  `a` bigint NOT NULL DEFAULT '0'
+  `a` bigint NOT NULL
 ) ENGINE=MyISAM
 drop table t1;
 create table t1 (a int) ENGINE=MyISAM;

=== modified file 'tests/r/subselect_no_mat.result'
--- tests/r/subselect_no_mat.result	2009-02-17 10:00:38 +0000
+++ tests/r/subselect_no_mat.result	2009-02-25 16:45:57 +0000
@@ -925,7 +925,7 @@
 CREATE TABLE `t1` (
 `mot` varchar(30) NOT NULL default '',
 `topic` bigint NOT NULL default '0',
-`date` date NULL,
+`date` date NULL DEFAULT '2009-01-20',
 `pseudo` varchar(35) NOT NULL default '',
 PRIMARY KEY  (`mot`,`pseudo`,`date`,`topic`),
 KEY `pseudo` (`pseudo`,`date`,`topic`),
@@ -934,7 +934,7 @@
 CREATE TABLE `t2` (
 `mot` varchar(30) NOT NULL default '',
 `topic` bigint NOT NULL default '0',
-`date` date NULL,
+`date` date NULL default '1997-08-29',
 `pseudo` varchar(35) NOT NULL default '',
 PRIMARY KEY  (`mot`,`pseudo`,`date`,`topic`),
 KEY `pseudo` (`pseudo`,`date`,`topic`),
@@ -955,13 +955,13 @@
 2
 select * from t1;
 mot	topic	date	pseudo
-joce	1	0000-00-00	joce
-test	2	0000-00-00	test
+joce	1	2009-01-20	joce
+test	2	2009-01-20	test
 DELETE FROM t1 WHERE topic IN (SELECT DISTINCT topic FROM t2 WHERE NOT
 EXISTS(SELECT * FROM t3 WHERE numeropost=topic));
 select * from t1;
 mot	topic	date	pseudo
-joce	1	0000-00-00	joce
+joce	1	2009-01-20	joce
 drop table t1, t2, t3;
 SELECT * FROM (SELECT 1 as a,(SELECT a)) a;
 a	(SELECT a)
@@ -997,7 +997,7 @@
 SHOW CREATE TABLE t1;
 Table	Create Table
 t1	CREATE TABLE `t1` (
-  `a` bigint NOT NULL DEFAULT '0'
+  `a` bigint NOT NULL
 ) ENGINE=MyISAM
 drop table t1;
 create table t1 (a int) ENGINE=MyISAM;

=== modified file 'tests/r/subselect_no_opts.result'
--- tests/r/subselect_no_opts.result	2009-02-17 10:00:38 +0000
+++ tests/r/subselect_no_opts.result	2009-02-25 16:45:57 +0000
@@ -925,7 +925,7 @@
 CREATE TABLE `t1` (
 `mot` varchar(30) NOT NULL default '',
 `topic` bigint NOT NULL default '0',
-`date` date NULL,
+`date` date NULL DEFAULT '2009-01-20',
 `pseudo` varchar(35) NOT NULL default '',
 PRIMARY KEY  (`mot`,`pseudo`,`date`,`topic`),
 KEY `pseudo` (`pseudo`,`date`,`topic`),
@@ -934,7 +934,7 @@
 CREATE TABLE `t2` (
 `mot` varchar(30) NOT NULL default '',
 `topic` bigint NOT NULL default '0',
-`date` date NULL,
+`date` date NULL default '1997-08-29',
 `pseudo` varchar(35) NOT NULL default '',
 PRIMARY KEY  (`mot`,`pseudo`,`date`,`topic`),
 KEY `pseudo` (`pseudo`,`date`,`topic`),
@@ -955,13 +955,13 @@
 2
 select * from t1;
 mot	topic	date	pseudo
-joce	1	0000-00-00	joce
-test	2	0000-00-00	test
+joce	1	2009-01-20	joce
+test	2	2009-01-20	test
 DELETE FROM t1 WHERE topic IN (SELECT DISTINCT topic FROM t2 WHERE NOT
 EXISTS(SELECT * FROM t3 WHERE numeropost=topic));
 select * from t1;
 mot	topic	date	pseudo
-joce	1	0000-00-00	joce
+joce	1	2009-01-20	joce
 drop table t1, t2, t3;
 SELECT * FROM (SELECT 1 as a,(SELECT a)) a;
 a	(SELECT a)
@@ -997,7 +997,7 @@
 SHOW CREATE TABLE t1;
 Table	Create Table
 t1	CREATE TABLE `t1` (
-  `a` bigint NOT NULL DEFAULT '0'
+  `a` bigint NOT NULL
 ) ENGINE=MyISAM
 drop table t1;
 create table t1 (a int) ENGINE=MyISAM;

=== modified file 'tests/r/subselect_no_semijoin.result'
--- tests/r/subselect_no_semijoin.result	2009-02-17 10:00:38 +0000
+++ tests/r/subselect_no_semijoin.result	2009-02-25 16:45:57 +0000
@@ -925,7 +925,7 @@
 CREATE TABLE `t1` (
 `mot` varchar(30) NOT NULL default '',
 `topic` bigint NOT NULL default '0',
-`date` date NULL,
+`date` date NULL DEFAULT '2009-01-20',
 `pseudo` varchar(35) NOT NULL default '',
 PRIMARY KEY  (`mot`,`pseudo`,`date`,`topic`),
 KEY `pseudo` (`pseudo`,`date`,`topic`),
@@ -934,7 +934,7 @@
 CREATE TABLE `t2` (
 `mot` varchar(30) NOT NULL default '',
 `topic` bigint NOT NULL default '0',
-`date` date NULL,
+`date` date NULL default '1997-08-29',
 `pseudo` varchar(35) NOT NULL default '',
 PRIMARY KEY  (`mot`,`pseudo`,`date`,`topic`),
 KEY `pseudo` (`pseudo`,`date`,`topic`),
@@ -955,13 +955,13 @@
 2
 select * from t1;
 mot	topic	date	pseudo
-joce	1	0000-00-00	joce
-test	2	0000-00-00	test
+joce	1	2009-01-20	joce
+test	2	2009-01-20	test
 DELETE FROM t1 WHERE topic IN (SELECT DISTINCT topic FROM t2 WHERE NOT
 EXISTS(SELECT * FROM t3 WHERE numeropost=topic));
 select * from t1;
 mot	topic	date	pseudo
-joce	1	0000-00-00	joce
+joce	1	2009-01-20	joce
 drop table t1, t2, t3;
 SELECT * FROM (SELECT 1 as a,(SELECT a)) a;
 a	(SELECT a)
@@ -997,7 +997,7 @@
 SHOW CREATE TABLE t1;
 Table	Create Table
 t1	CREATE TABLE `t1` (
-  `a` bigint NOT NULL DEFAULT '0'
+  `a` bigint NOT NULL
 ) ENGINE=MyISAM
 drop table t1;
 create table t1 (a int) ENGINE=MyISAM;

=== modified file 'tests/r/type_blob.result'
--- tests/r/type_blob.result	2009-02-17 10:00:38 +0000
+++ tests/r/type_blob.result	2009-02-25 16:45:57 +0000
@@ -28,11 +28,13 @@
 ) ENGINE=InnoDB
 drop table t1,t2,t3,t4;
 CREATE TABLE t1 (a blob default "hello");
+ERROR 42000: BLOB/TEXT column 'a' can't have a default value
 CREATE TABLE t2 (a varchar(256));
-drop table t1,t2;
+drop table t2;
 CREATE TABLE t1 (a varchar(70000) default "hello");
 ERROR 42000: Column length too big for column 'a' (max = 65535); use BLOB or TEXT instead
 CREATE TABLE t2 (a blob default "hello");
+ERROR 42000: BLOB/TEXT column 'a' can't have a default value
 drop table if exists t1,t2;
 create table t1 (nr int not null auto_increment,b blob,str char(10), primary key (nr));
 insert into t1 values (null,"a","A");
@@ -598,10 +600,10 @@
 show create table t1;
 Table	Create Table
 t1	CREATE TABLE `t1` (
-  `a` int NOT NULL DEFAULT '0',
-  `b` int NOT NULL DEFAULT '0',
+  `a` int NOT NULL,
+  `b` int NOT NULL,
   `c` blob NOT NULL,
-  `d` int NOT NULL DEFAULT '0',
+  `d` int NOT NULL,
   `e` int DEFAULT NULL,
   PRIMARY KEY (`a`,`b`,`c`(),`d`),
   KEY `a` (`a`,`b`,`d`,`e`)

=== modified file 'tests/r/type_enum.result'
--- tests/r/type_enum.result	2009-02-17 10:00:38 +0000
+++ tests/r/type_enum.result	2009-02-25 16:45:57 +0000
@@ -1739,18 +1739,19 @@
 CREATE TABLE t1 (c1 ENUM('a', '', 'b'));
 INSERT INTO t1 (c1) VALUES ('b');
 INSERT INTO t1 (c1) VALUES ('');
-ERROR 01000: Data truncated for column 'c1' at row 1
 INSERT INTO t1 (c1) VALUES (0);
 ERROR 01000: Data truncated for column 'c1' at row 1
 INSERT INTO t1 (c1) VALUES ('a');
 SELECT c1 + 0, COUNT(c1) FROM t1 GROUP BY c1;
 c1 + 0	COUNT(c1)
 1	1
+2	1
 3	1
 CREATE TABLE t2 SELECT * FROM t1;
 SELECT c1 + 0 FROM t2;
 c1 + 0
 3
+2
 1
 DROP TABLE t1,t2;
 CREATE TABLE t1(a enum('a','b','c','d'));

=== modified file 'tests/r/type_float.result'
--- tests/r/type_float.result	2009-02-17 10:00:38 +0000
+++ tests/r/type_float.result	2009-02-25 16:45:57 +0000
@@ -209,22 +209,22 @@
 Level	Code	Message
 desc  t1;
 Field	Type	Null	Key	Default	Extra
-x	decimal(21,2)	NO		0.00	
+x	decimal(21,2)	NO		NULL	
 drop table t1;
 create table t1 select 0.0 x;
 desc t1;
 Field	Type	Null	Key	Default	Extra
-x	decimal(2,1)	NO		0.0	
+x	decimal(2,1)	NO		NULL	
 create table t2 select 105213674794682365.00 y;
 desc t2;
 Field	Type	Null	Key	Default	Extra
-y	decimal(20,2)	NO		0.00	
+y	decimal(20,2)	NO		NULL	
 create table t3 select x+y a from t1,t2;
 show warnings;
 Level	Code	Message
 desc t3;
 Field	Type	Null	Key	Default	Extra
-a	decimal(21,2)	NO		0.00	
+a	decimal(21,2)	NO		NULL	
 drop table t1,t2,t3;
 select 1e-308, 1.00000001e-300, 100000000e-300;
 1e-308	1.00000001e-300	100000000e-300

=== modified file 'tests/r/type_newdecimal.result'
--- tests/r/type_newdecimal.result	2009-02-17 10:00:38 +0000
+++ tests/r/type_newdecimal.result	2009-02-25 16:45:57 +0000
@@ -52,13 +52,13 @@
 show create table t1;
 Table	Create Table
 t1	CREATE TABLE `t1` (
-  `if(1, 1.1, 1.2)` decimal(2,1) NOT NULL DEFAULT '0.0',
-  `if(0, 1.1, 1.2)` decimal(2,1) NOT NULL DEFAULT '0.0',
-  `if(0.1, 1.1, 1.2)` decimal(2,1) NOT NULL DEFAULT '0.0',
-  `if(0, 1, 1.1)` decimal(2,1) NOT NULL DEFAULT '0.0',
+  `if(1, 1.1, 1.2)` decimal(2,1) NOT NULL,
+  `if(0, 1.1, 1.2)` decimal(2,1) NOT NULL,
+  `if(0.1, 1.1, 1.2)` decimal(2,1) NOT NULL,
+  `if(0, 1, 1.1)` decimal(2,1) NOT NULL,
   `if(0, NULL, 1.2)` decimal(2,1) DEFAULT NULL,
-  `if(1, 0.22e1, 1.1)` double NOT NULL DEFAULT '0',
-  `if(1E0, 1.1, 1.2)` decimal(2,1) NOT NULL DEFAULT '0.0'
+  `if(1, 0.22e1, 1.1)` double NOT NULL,
+  `if(1E0, 1.1, 1.2)` decimal(2,1) NOT NULL
 ) ENGINE=X
 drop table t1;
 create table t1 select nullif(1.1, 1.1), nullif(1.1, 1.2), nullif(1.1, 0.11e1), nullif(1.0, 1), nullif(1, 1.0), nullif(1, 1.1);
@@ -124,10 +124,10 @@
 show create table t1;
 Table	Create Table
 t1	CREATE TABLE `t1` (
-  `round(15.4,-1)` decimal(3,0) NOT NULL DEFAULT '0',
-  `truncate(-5678.123451,-3)` decimal(4,0) NOT NULL DEFAULT '0',
-  `abs(-1.1)` decimal(3,1) NOT NULL DEFAULT '0.0',
-  `-(-1.1)` decimal(2,1) NOT NULL DEFAULT '0.0'
+  `round(15.4,-1)` decimal(3,0) NOT NULL,
+  `truncate(-5678.123451,-3)` decimal(4,0) NOT NULL,
+  `abs(-1.1)` decimal(3,1) NOT NULL,
+  `-(-1.1)` decimal(2,1) NOT NULL
 ) ENGINE=X
 drop table t1;
 select 1e10/0e0;
@@ -719,7 +719,7 @@
 show create table t1;
 Table	Create Table
 t1	CREATE TABLE `t1` (
-  `0.5` decimal(2,1) NOT NULL DEFAULT '0.0'
+  `0.5` decimal(2,1) NOT NULL
 ) ENGINE=X
 drop table t1;
 select round(1.5),round(2.5);
@@ -1408,7 +1408,7 @@
 Note	1265	Data truncated for column 'f1' at row 1
 DESC t1;
 Field	Type	Null	Key	Default	Extra
-f1	decimal(31,30)	NO		0.000000000000000000000000000000	
+f1	decimal(31,30)	NO		NULL	
 SELECT f1 FROM t1;
 f1
 0.123456789012345678901234567890

=== modified file 'tests/r/type_ranges.result'
--- tests/r/type_ranges.result	2009-02-17 10:00:38 +0000
+++ tests/r/type_ranges.result	2009-02-25 16:45:57 +0000
@@ -190,14 +190,14 @@
 create table t2 (primary key (auto)) select auto+1 as auto,1 as t1, 'a' as t2, repeat('a',256) as t3, binary repeat('b',256) as t4, repeat('a',4096) as t5, binary repeat('b',4096) as t6, '' as t7, binary '' as t8 from t1;
 show full columns from t2;
 Field	Type	Collation	Null	Key	Default	Extra	Privileges	Comment
-auto	bigint	NULL	NO	PRI	0		#	
-t1	int	NULL	NO		0		#	
-t2	varchar(1)	utf8_general_ci	NO				#	
+auto	bigint	NULL	NO	PRI	NULL		#	
+t1	int	NULL	NO		NULL		#	
+t2	varchar(1)	utf8_general_ci	NO		NULL		#	
 t3	varchar(256)	utf8_general_ci	YES		NULL		#	
 t4	varbinary(256)	NULL	YES		NULL		#	
 t5	text	utf8_general_ci	YES		NULL		#	
 t6	blob	NULL	YES		NULL		#	
-t7	varchar(0)	utf8_general_ci	NO				#	
+t7	varchar(0)	utf8_general_ci	NO		NULL		#	
 t8	varbinary(0)	NULL	YES		NULL		#	
 select t1,t2,length(t3),length(t4),length(t5),length(t6),t7,t8 from t2;
 t1	t2	length(t3)	length(t4)	length(t5)	length(t6)	t7	t8
@@ -215,7 +215,7 @@
 Field	Type	Collation	Null	Key	Default	Extra	Privileges	Comment
 c1	int	NULL	YES		NULL		#	
 c2	int	NULL	YES		NULL		#	
-const	int	NULL	NO		0		#	
+const	int	NULL	NO		NULL		#	
 drop table t1,t2,t3;
 create table t1 ( myfield INT NOT NULL, UNIQUE INDEX (myfield), unique (myfield), index(myfield));
 drop table t1;

=== modified file 'tests/t/alter_table.test'
--- tests/t/alter_table.test	2008-12-06 22:41:03 +0000
+++ tests/t/alter_table.test	2009-02-25 16:45:57 +0000
@@ -250,7 +250,7 @@
 # not null columns for primary keys)
 
 create table t1 (a int, b int, c int, d int, e int, f int, g int, h int,i int, primary key (a,b,c,d,e,f,g,i,h)) engine=MyISAM;
-insert into t1 (a) values(1);
+insert into t1 (a,b,c,d,e,f,g,h,i) values(1,1,1,1,1,1,1,1,1);
 --replace_column 3 X 7 X 8 X 9 X 10 X 11 X 12 X 13 X 14 X
 show table status like 't1';
 alter table t1 modify a int;

=== modified file 'tests/t/distinct.test'
--- tests/t/distinct.test	2009-02-11 02:30:38 +0000
+++ tests/t/distinct.test	2009-02-25 16:45:57 +0000
@@ -469,7 +469,7 @@
 #
 # Bug #34928: Confusion by having Primary Key and Index
 #
-CREATE TABLE t1(a INT, b INT, c INT, d INT, e INT,
+CREATE TABLE t1(a INT, b INT, c INT, d INT DEFAULT 0, e INT DEFAULT 0,
                 PRIMARY KEY(a,b,c,d,e),
                 KEY(a,b,d,c)
 );

=== modified file 'tests/t/subselect.test'
--- tests/t/subselect.test	2009-02-11 04:11:47 +0000
+++ tests/t/subselect.test	2009-02-25 16:45:57 +0000
@@ -521,7 +521,7 @@
 CREATE TABLE `t1` (
   `mot` varchar(30) NOT NULL default '',
   `topic` bigint NOT NULL default '0',
-  `date` date NULL,
+  `date` date NULL DEFAULT '2009-01-20',
   `pseudo` varchar(35) NOT NULL default '',
   PRIMARY KEY  (`mot`,`pseudo`,`date`,`topic`),
   KEY `pseudo` (`pseudo`,`date`,`topic`),
@@ -531,7 +531,7 @@
 CREATE TABLE `t2` (
   `mot` varchar(30) NOT NULL default '',
   `topic` bigint NOT NULL default '0',
-  `date` date NULL,
+  `date` date NULL default '1997-08-29',
   `pseudo` varchar(35) NOT NULL default '',
   PRIMARY KEY  (`mot`,`pseudo`,`date`,`topic`),
   KEY `pseudo` (`pseudo`,`date`,`topic`),

=== modified file 'tests/t/type_blob.test'
--- tests/t/type_blob.test	2009-02-11 04:11:47 +0000
+++ tests/t/type_blob.test	2009-02-25 16:44:44 +0000
@@ -29,11 +29,13 @@
 # Check errors with blob
 #
 
+--error 1101
 CREATE TABLE t1 (a blob default "hello");
 CREATE TABLE t2 (a varchar(256));
-drop table t1,t2;
+drop table t2;
 --error 1074
 CREATE TABLE t1 (a varchar(70000) default "hello");
+--error 1101
 CREATE TABLE t2 (a blob default "hello");
 
 # Safety to be able to continue with other tests if above fails

=== modified file 'tests/t/type_enum.test'
--- tests/t/type_enum.test	2008-12-24 02:21:17 +0000
+++ tests/t/type_enum.test	2009-02-25 16:45:57 +0000
@@ -186,7 +186,6 @@
 
 CREATE TABLE t1 (c1 ENUM('a', '', 'b'));
 INSERT INTO t1 (c1) VALUES ('b');
---error 1265
 INSERT INTO t1 (c1) VALUES ('');
 --error 1265
 INSERT INTO t1 (c1) VALUES (0);

